/**********************************************************************/
/* McLib: Library for Many-Core Research Env.    Arch Lab. TOKYO TECH */
/**********************************************************************/

#ifdef SC
#include "sc/MClib.h"
#else

#ifndef _MCLIB_H_
#define _MCLIB_H_

#include <stdio.h>
#include <stdarg.h>

/************************************************************************/
/* MieruPC Memory Map : 0x100 - 0x1ff                    Ver.2009-06-19 */
/************************************************************************/
#define MP_MMAP_BASE                   0x00000100

#define MP_MMAP_LCD                    (MP_MMAP_BASE + 0x04)
#define MP_MMAP_TIMER                  (MP_MMAP_BASE + 0x0c)

/************************************************************************/
/* M-Core Memory Map : 0x200 - 0x3ff                     Ver.2012-09-18 */
/************************************************************************/
#define SC_MMAP_BASE                   0x00000200

#define SC_MMAP_NODE_ID                (SC_MMAP_BASE + 0x00)
#define SC_MMAP_NOC_SIZE               (SC_MMAP_BASE + 0x04)

#define SC_MMAP_PC                     (SC_MMAP_BASE + 0x08)
#define SC_MMAP_TIMER                  (SC_MMAP_BASE + 0x10) // clock
#define SC_MMAP_TIMER_D                (SC_MMAP_BASE + 0x14) // 

#define SC_MMAP_INST_LOOP              (SC_MMAP_BASE + 0x18) // j
#define SC_MMAP_INST_LOOP__            (SC_MMAP_BASE + 0x1c) // nop

#define SC_MMAP_DMA_PUT_REMOTE_ID      (SC_MMAP_BASE + 0x20)
#define SC_MMAP_DMA_PUT_REMOTE_ADDR    (SC_MMAP_BASE + 0x24)
#define SC_MMAP_DMA_PUT_REMOTE_STRIDE  (SC_MMAP_BASE + 0x28) // or addr_h
#define SC_MMAP_DMA_PUT_LOCAL_ADDR     (SC_MMAP_BASE + 0x2c)
#define SC_MMAP_DMA_PUT_LOCAL_STRIDE   (SC_MMAP_BASE + 0x30)
#define SC_MMAP_DMA_PUT_SIZE           (SC_MMAP_BASE + 0x34)
//      unused                         (SC_MMAP_BASE + 0x38)
#define SC_MMAP_DMA_PUT_CMD            (SC_MMAP_BASE + 0x3c)

#define SC_MMAP_DMA_GET_REMOTE_ID      (SC_MMAP_BASE + 0x40)
#define SC_MMAP_DMA_GET_LOCAL_ADDR     (SC_MMAP_BASE + 0x44)
#define SC_MMAP_DMA_GET_LOCAL_STRIDE   (SC_MMAP_BASE + 0x48)
#define SC_MMAP_DMA_GET_REMOTE_ADDR    (SC_MMAP_BASE + 0x4c)
#define SC_MMAP_DMA_GET_REMOTE_STRIDE  (SC_MMAP_BASE + 0x50)
#define SC_MMAP_DMA_GET_SIZE           (SC_MMAP_BASE + 0x54)
//      unused                         (SC_MMAP_BASE + 0x58)
#define SC_MMAP_DMA_GET_CMD            (SC_MMAP_BASE + 0x5c)

#define SC_MMAP_DMA_FLAG               (SC_MMAP_BASE + 0x80) // for DMA_get
#define SC_MMAP_DMA_PUT_CH             (SC_MMAP_BASE + 0x84) 
#define SC_MMAP_DMA_GET_CH             (SC_MMAP_BASE + 0x88) 

/**********************************************************************/
/* Memory-Node Memory Map (64-bit local address, no stride support)   */
/**********************************************************************/
#define GMEM_BASE                  0x00000200

#define GMEM_DMA_GET_REMOTE_ID     (GMEM_BASE + 0x40)
#define GMEM_DMA_GET_REMOTE_ADDR   (GMEM_BASE + 0x44)
#define GMEM_DMA_GET_REMOTE_STRIDE (GMEM_BASE + 0x48)
#define GMEM_DMA_GET_LOCAL_ADDR_L  (GMEM_BASE + 0x4c)
#define GMEM_DMA_GET_LOCAL_ADDR_H  (GMEM_BASE + 0x50)
#define GMEM_DMA_GET_SIZE          (GMEM_BASE + 0x54)
//      unused                     (GMEM_BASE + 0x58)
#define GMEM_DMA_GET_CMD           (GMEM_BASE + 0x5c)

/**********************************************************************/
/* for Lock/Barrier Server Node Routine (not memory-mapped I/O)       */
/**********************************************************************/
#define MEM_BARRIER_WORKER_FLAG   (GMEM_BASE + 0xe0)    // 4byte
#define MEM_LOCK_WORKER_FLAG      (GMEM_BASE + 0xe8)    // 4byte
#define MEM_LOCK_SERVER_FLAG      (GMEM_BASE + 0xec)    // 4byte

#define MEM_BARRIER_SERVER_BASE   (GMEM_BASE + 0x10000) // 4*cores
#define MEM_LOCK_SERVER_BASE      (GMEM_BASE + 0x20000) // 4*cores

#define CORE_LOCATE_BARRIER_X 1  // x location of the barrier server
#define CORE_LOCATE_LOCK_X    2  // x location of the lock server
#define CORE_LOADER           4

/********************************************************************/

typedef unsigned int uint032_t;
typedef unsigned long long int uint064_t;

enum { // DMA Flag (Put)
    DMA_NA = 0,
    DMA_PUT = 1,
    DMA_PUT64 = 2,
};

enum { // DMA Flag (Get)
    DMAFLAG_NA,
    DMAFLAG_ACK,
    DMAFLAG_NACK,
};

/********************************************************************/
inline void setidx(int *id, int x);
inline void setidy(int *id, int y);
inline void setidxy(int *id, int x, int y);
inline int  getidx(int id);
inline int  getidy(int id);
void        new_barrier();

/********************************************************************/
inline int  MC_barrier();
int         MC_server_barrier();
inline int  MC_lock(int lock);
inline int  MC_unlock(int lock);
int         MC_server_lock();
int         MC_server_loader();
inline void MC_setidx(int *id, int x);
inline void MC_setidy(int *id, int y);
inline void MC_setidxy(int *id, int x, int y);
inline int  MC_getidx(int id);
inline int  MC_getidy(int id);
inline int  MC_get_id(int *id);
inline int  MC_get_rank(int *rank);
inline int  MC_get_physical_id(int *id);
inline void MC_initialize();
int         MC_init(int *id_x, int *id_y, int *rank_x, int *rank_y);
void        MC_finalize();
void        MC_fini();
int         MC_dma_put_enable();
void        MC_dma_put(int remote_id, void *remote, void *local, size_t size,
                       int remote_stride, int local_stride);
void        MC_dma_put_nowait(int remote_id, void *remote, void *local,
                              size_t size, int remote_stride, int local_stride);
void        MC_dma_puts(void *remote, void *local, size_t size,
                        int remote_stride, int local_stride);
void        MC_dma_put64(int remote_id, uint064_t remote, void *local,
                         size_t size, int local_stride);
void        MC_dma_put_4b(int remote_id, void *addr, int data);
void        MC_dma_put_4b_nowait(int remote_id, void *addr, int data);
void        MC_dma_get(int remote_id,
                       volatile void *remote, volatile void *local,
                       size_t size, int remote_stride, int local_stride);
void        MC_dma_get64(int remote_id, 
                         uint064_t remote, volatile void *local,
                         size_t size, int local_stride);
int         MC_sleep(long long unsigned int n);
int         MC_clock(long long unsigned int *);
int         MC_fast_barrier();
int         MC_infloop();
void        MC_new_barrier();
void        MC_gettime(unsigned int *t);

/********************************************************************/
#endif
#endif
