/**********************************************************************/
/* SimMc: Simulator for Many-Core Architecture Research Environment   */
/* Copyright (C) 2010-2013 Arch Lab. Tokyo Institute of Technology    */
/*  Licensed under GPL. See COPYING file for more information.        */
/**********************************************************************/
#pragma once

#include "mapparser.h"
#include "inttypes.h"
#include "id.h"
#include <map>
#include <string>

/**********************************************************************/
class SystemConfig {
 private:
    std::map<uint, int>  patable;  // physical ID -> app ID
    std::map<uint, uint> pltable;  // physical ID -> logical ID
    std::vector<std::map<uint, uint> > lptable, lpslave;
                                   // app ID + logical ID -> physical ID
    std::map<uint, uint> mstable;  // Master pID -> Slave  pID
    std::map<uint, uint> smtable;  // Slave  pID -> Master pID
    void registerdefaultid(uint pid, uint lid, uint rank, bool smart,
                           std::string pattern = "INTERLEAVE");
    void getsmartcorepids(uint pid, uint rank, std::string &pattern,
                          uint &pid_m, uint &pid_s);

 public:
    int x_rank, y_rank;            // Rank X/Y of the System
    std::vector<int> x_app, y_app; // Rank X/Y of the Application
    int num_core;                  // # of Started Cores
    int num_app;                   // # of Applications

    bool lookahead_rc;             // Look-ahead Route Computation
    bool fix_vc;                   // fix input VC = output VC
    bool fix_priority;             // fix arbiter priority
    bool credit_fc;                // enable Credit-base flow control
    bool skip_lt;                  // skip LT stage
    int num_channel, num_inbuf;    // # of Channels, Input Buffers
    int max_packet_length;         // maximum packet length (for SmartUnit)

    SystemConfig() { clear(); }
    void clear();
    void cleartable();
    bool readconfigfile(const char *);
    bool setdefaultconfig(int x, int y, bool mem = false, 
                          bool server = false, bool smart = false,
                          std::string pattern = "INTERLEAVE");

    bool isvalid(uint pid)  { return (pltable.find(pid) != pltable.end()); }
    bool ismaster(uint pid) { return (mstable.find(pid) != mstable.end()); }
    bool isslave(uint pid)  { return (smtable.find(pid) != smtable.end()); }
    uint getpid(int app, uint lid) { return lptable[app][lid]; }
    int  getaid(uint pid)          { return patable[pid]; }
    uint getlid(uint pid)          { return pltable[pid]; }
    uint getprank()                { return ID(x_rank, y_rank); }
    uint getlrank(int app)         { return ID(x_app[app], y_app[app]); }
    uint getmasterpid(uint pid)    { return smtable[pid]; }
    uint getslavepid(uint pid)     { return mstable[pid]; }
    std::map<uint, uint>& getlptable(int app) { return lptable[app]; }
    void print();
};

/**********************************************************************/
