/**********************************************************************/
/* SimMc: Simulator for Many-Core Architecture Research Environment   */
/* Copyright (C) 2010-2013 Arch Lab. Tokyo Institute of Technology    */
/*  Licensed under GPL. See COPYING file for more information.        */
/**********************************************************************/
#pragma once

#include "inttypes.h"
#include <stdio.h>

/* Main Memory                                                        */
/**********************************************************************/
class MainMemory {
 protected:
    static const uint064_t MEM_SIZE_DEF   = 0x100000000ull;
    static const uint064_t PAGE_SIZE_DEF  = 4096;
    static const uint064_t LPAGE_SIZE_DEF = PAGE_SIZE_DEF * 4096;

    bool use_lpage;
    uint064_t mem_size, npage, nlpage, page_size, lpage_size;
    uint032_t ***lpagetable;
    uint032_t **pagetable;
    void initpagetable();
    uint032_t **getlpage(const uint064_t);
    uint032_t *getpage(const uint064_t);
    uint032_t *getrealaddr(const uint064_t);
    
 public:
    MainMemory();
    ~MainMemory() { clear(); };
    void init(ullint);
    void clear();
    void drive() {};
    void update() {};
    int read(const uint064_t, uint, void*);
    int write(const uint064_t, uint, uint064_t);
    int readnow(const uint064_t a, uint s, void* d)
        { return read(a, s, d); }
    int writenow(const uint064_t a, uint s, uint064_t d)
        { return write(a, s, d); }
    int isbusy() { return 0; }
    void print();
};

/**********************************************************************/
