/**********************************************************************/
/* SimMc: Simulator for Many-Core Architecture Research Environment   */
/* Copyright (C) 2010-2013 Arch Lab. Tokyo Institute of Technology    */
/*  Licensed under GPL. See COPYING file for more information.        */
/**********************************************************************/
#pragma once

#include <string>
#include <iostream>
#include <fstream>
#include <sstream>
#include <vector>

/* Physical to Logical Core Mapping ***********************************/
/**********************************************************************/
class PLMap {
 public:
    bool valid;
    int app, x, y;
    bool slave;

    PLMap() { valid = false; }
    PLMap(int x, int y, int app = 0, bool slave = false) {
        valid = true;
        this->x     = x;
        this->y     = y;
        this->app   = app;
        this->slave = slave;
    }
    bool operator== (PLMap &obj) {
        return (this->valid == obj.valid &&
                this->x     == obj.x     && this->y     == obj.y     &&
                this->app   == obj.app   && this->slave == obj.slave);
    }
    void print(std::ostream &out = std::cout) {
        if (valid)
            out << '[' << (char) ('a' + app) << x << ',' << y 
                <<  ((slave) ? 's' : 'm') << "] ";
        else
            out << "[-----] ";
    }
};

/* Mapping File Parser ************************************************/
/**********************************************************************/
class MapParser {
 private:
    std::ostringstream errstr;
    std::vector<PLMap> parseline(std::istringstream &, bool);
    void parsesize(std::istringstream &, bool);
    bool sanitycheck();

 public:
    bool valid;
    int x_rank, y_rank;
    std::vector<int> x_app, y_app;
    std::vector<PLMap> maps;

    MapParser() { clear(); }
    void clear();
    void parse(const char *);
    void print();
    void puterror() { std::cout << errstr.str(); }
};

/**********************************************************************/
