/**********************************************************************/
/* SimMc: Simulator for Many-Core Architecture Research Environment   */
/* Copyright (C) 2010-2013 Arch Lab. Tokyo Institute of Technology    */
/*  Licensed under GPL. See COPYING file for more information.        */
/**********************************************************************/
#pragma once

/**********************************************************************/
/* MieruPC Memory Map : 0x100 - 0x1ff                  Ver.2009-06-19 */
/**********************************************************************/
#define MP_MMAP_BASE                   0x00000100

#define MP_MMAP_LCD                    (MP_MMAP_BASE + 0x04)
#define MP_MMAP_TIMER                  (MP_MMAP_BASE + 0x0c)

/**********************************************************************/
/* ScalableCore Memory Map : 0x200 - 0x3ff             Ver.2012-09-18 */
/**********************************************************************/
#define SC_MMAP_BASE                   0x00000200

#define SC_MMAP_NODE_ID                (SC_MMAP_BASE + 0x00)
#define SC_MMAP_NOC_SIZE               (SC_MMAP_BASE + 0x04)

#define SC_MMAP_PC                     (SC_MMAP_BASE + 0x08)
#define SC_MMAP_TIMER                  (SC_MMAP_BASE + 0x10) // clock
#define SC_MMAP_TIMER_D                (SC_MMAP_BASE + 0x14) // 

#define SC_MMAP_INST_LOOP              (SC_MMAP_BASE + 0x18) // j
#define SC_MMAP_INST_LOOP__            (SC_MMAP_BASE + 0x1c) // nop

#define SC_MMAP_DMA_PUT_REMOTE_ID      (SC_MMAP_BASE + 0x20)
#define SC_MMAP_DMA_PUT_REMOTE_ADDR    (SC_MMAP_BASE + 0x24)
#define SC_MMAP_DMA_PUT_REMOTE_STRIDE  (SC_MMAP_BASE + 0x28) // or addr_h
#define SC_MMAP_DMA_PUT_LOCAL_ADDR     (SC_MMAP_BASE + 0x2c)
#define SC_MMAP_DMA_PUT_LOCAL_STRIDE   (SC_MMAP_BASE + 0x30)
#define SC_MMAP_DMA_PUT_SIZE           (SC_MMAP_BASE + 0x34)
//      unused                         (SC_MMAP_BASE + 0x38)
#define SC_MMAP_DMA_PUT_CMD            (SC_MMAP_BASE + 0x3c)

#define SC_MMAP_DMA_GET_REMOTE_ID      (SC_MMAP_BASE + 0x40)
#define SC_MMAP_DMA_GET_LOCAL_ADDR     (SC_MMAP_BASE + 0x44)
#define SC_MMAP_DMA_GET_LOCAL_STRIDE   (SC_MMAP_BASE + 0x48)
#define SC_MMAP_DMA_GET_REMOTE_ADDR    (SC_MMAP_BASE + 0x4c)
#define SC_MMAP_DMA_GET_REMOTE_STRIDE  (SC_MMAP_BASE + 0x50)
#define SC_MMAP_DMA_GET_SIZE           (SC_MMAP_BASE + 0x54)
//      unused                         (SC_MMAP_BASE + 0x58)
#define SC_MMAP_DMA_GET_CMD            (SC_MMAP_BASE + 0x5c)

#define SC_MMAP_DMA_FLAG               (SC_MMAP_BASE + 0x80)
#define SC_MMAP_DMA_PUT_CH             (SC_MMAP_BASE + 0x84) // add
#define SC_MMAP_DMA_GET_CH             (SC_MMAP_BASE + 0x88) // 2012-09

/**********************************************************************/
/* Memory-Node Memory Map (64-bit local address, no stride support)   */
/**********************************************************************/
#define GMEM_BASE                  0x00000200

#define GMEM_DMA_GET_REMOTE_ID     (GMEM_BASE + 0x40)
#define GMEM_DMA_GET_REMOTE_ADDR   (GMEM_BASE + 0x44)
#define GMEM_DMA_GET_REMOTE_STRIDE (GMEM_BASE + 0x48)
#define GMEM_DMA_GET_LOCAL_ADDR_L  (GMEM_BASE + 0x4c)
#define GMEM_DMA_GET_LOCAL_ADDR_H  (GMEM_BASE + 0x50)
#define GMEM_DMA_GET_SIZE          (GMEM_BASE + 0x54)
//      unused                     (GMEM_BASE + 0x58)
#define GMEM_DMA_GET_CMD           (GMEM_BASE + 0x5c)

/**********************************************************************/
