/**********************************************************************
 * SimLoader: Executable File Loader for Processor Simulators         *
 *  as a part of SimMips: Simple Computer Simulator of MIPS           *
 * Copyright (C) 2008-2012 Arch Lab. Tokyo Institute of Technology    *
 *  Dual licensed under New BSD or GPL v2 licenses.                   *
 *  See COPYING file on the base directory for more information.      *
 **********************************************************************/
#pragma once

#include <fstream>
#include <vector>
#include <string>
#include <elf.h>

enum {
    ST_NOTYPE = 0,
    ST_FUNC   = 1,
    ST_OBJECT = 2,
    FI_LITTLE = 0,
    FI_BIG    = 1,
};

/**********************************************************************/
class SimLoader {
 private:
    bool checkfile(std::ifstream &);
    bool loadelf32(std::ifstream &);

 public:
    typedef struct {
        unsigned int  addr;
        unsigned char data;
    } memtab_t;

    typedef struct {
        unsigned int addr;
        int          type;
        char        *name;
        std::string namestr;
    } symtab_t;

    std::vector<memtab_t> memtab;
    std::vector<symtab_t> symtab;
    
    int filetype;
    int endian;
    int archtype;
    int dynamic;
    unsigned int entry;
    unsigned int stackptr;
    int memtabnum;
    int symtabnum;
    unsigned int phoff, phsize, phnum, phstart;
    
    SimLoader();
    int loadfile(const char *);
    unsigned int getSymbolAddress(const char *);
};

/**********************************************************************/
