/**********************************************************************
 * SimMips: Simple Computer Simulator of MIPS                         *
 * Copyright (C) 2008-2012 Arch Lab. Tokyo Institute of Technology    *
 *  Dual licensed under New BSD or GPL v2 licenses.                   *
 *  See COPYING file on the base directory for more information.      *
 **********************************************************************/
#pragma once

#define L_NAME_SYSCALL "System Call Expansion"
#define L_VER_SYSCALL  "2012-09-07"
#define NUM_FILENAME_BYTES  256
#define NUM_IOCTL_BYTES  128
#define FREQUENCY 10000000

#define MemoryController MainMemory

/* Options ************************************************************/
#define BASE_MODE_FLOAT
//#define STRACE
#ifdef _clang_
#define SCPLUS_NO_RESOURCE
#define SCPLUS_NO_TIMES
#define SCPLUS_NO_IOCTL
#define SCPLUS_NO_GETDENTS
#endif

/**********************************************************************/
#include <map>
#include <vector>
#include <string>
#include <iostream>
#include <iomanip>

#include <stdio.h>
#include <unistd.h>
#include <stdarg.h>
#include <string.h>
#include <errno.h>
#include <fcntl.h>
#include <utime.h>
#include <sys/stat.h>
#include <sys/time.h>
#ifndef SCPLUS_NO_RESOURCE
#include <sys/resource.h>
#endif
#ifndef SCPLUS_NO_TIMES
#include <sys/times.h>
#else
struct tms {};
#endif
#ifndef SCPLUS_NO_IOCTL
#include <sys/ioctl.h>
#endif
#ifndef SCPLUS_NO_GETDENTS
#include <asm/unistd.h>
#include <dirent.h>
#endif

#include "../common/inttypes.h"
#include "../common/mainmemory.h"

#define FILENAME_BUF_SIZE 256

/**********************************************************************/
#define LOCAL_TIOCGETP   0
#define LOCAL_TIOCSETP   0
#define LOCAL_TCGETA     0
#define LOCAL_TIOCSLTC   0
#define LOCAL_TIOCGWINSZ 0
#define LOCAL_TCSETAW    0

#ifdef TIOCGETP
#undef  LOCAL_TIOCGETP
#define LOCAL_TIOCGETP   TIOCGETP
#endif
#ifdef TIOCSETP
#undef  LOCAL_TIOCSETP
#define LOCAL_TIOCSETP   TIOCSETP
#endif
#ifdef TCGETA
#undef  LOCAL_TCGETA
#define LOCAL_TCGETA     TCGETA
#endif
#ifdef TIOCSLTC
#undef  LOCAL_TIOCSLTC
#define LOCAL_TIOCSLTC   TIOCSLTC
#endif
#ifdef TIOCGWINSZ
#undef  LOCAL_TIOCGWINSZ
#define LOCAL_TIOCGWINSZ TIOCGWINSZ
#endif
#ifdef TCSETAW
#undef  LOCAL_TCSETAW
#define LOCAL_TCSETAW    TCSETAW
#endif

/**********************************************************************/
class HeapAllocator {
 private:
    std::vector<uint> table;

 public:
    uint032_t heapptr, mmapptr;

    void init(uint032_t, uint032_t);
    uint032_t extendheap(uint032_t);
    uint032_t findmap(uint032_t, uint032_t, uint032_t);
    bool remap(uint032_t, uint032_t, uint032_t);
};

/**********************************************************************/
class SystemCallEmulator {
 private:
    typedef struct {
        std::string cwd;
        HeapAllocator mm;
        uint064_t startcycle, basecycle, basecycle_tms, basetime;
        struct tms basetms;
        uint pid, uid, euid, gid, egid;
        std::map<int, int> fdfd;
        std::vector<std::vector<uint032_t> > log;
    } appspecific_t;

    static const char *sc_list[];
    std::map<int, std::string> sc_name, sc_arg;
    static const int sc_conv_open[], sc_conv_ioctl[];

    std::string sim_cwd;
    appspecific_t app;
    MemoryController *mc;
    int kind, syscall_errno;
    int a[6];
    std::string a_str[1];
    int a_fd;

    int syscall_internal(uint032_t *r, uint064_t);
    void initappspecific();
    bool memstrcpy(uint008_t *, uint064_t);  // buffer <- memory
    bool memmemcpy(void *, uint064_t, uint); // buffer <- memory
    bool memmemcpy(uint064_t, void *, uint); // memory <- buffer
    bool memread(uint032_t a, uint s, void *d)
        { return (mc->readnow(a, s, d)  != -1); } 
    bool memwrite(uint032_t a, uint s, uint064_t d)
        { return (mc->writenow(a, s, d) != -1); }
    int getfd(int);
    void setcwd();
    void restorecwd() { int x = chdir(sim_cwd.c_str()); x = x; }
    void updatecwd();
    void addlog(int, ...);
    void addlog(std::vector<uint032_t> newvec) 
        { app.log.push_back(newvec); }
    void pushstrtolog(std::vector<uint032_t> & vec, std::string str)
        { for (uint i = 0; i < str.length(); i++) vec.push_back(str[i]); }
    void puttrace(uint032_t *);

 public:
    bool initialized;
    bool strace_mode, silent_mode;

    SystemCallEmulator() { initialized = false; }
    void init (MemoryController *, uint032_t, uint032_t);
    void syscall (uint032_t *, uint064_t);
    std::vector<std::vector<uint032_t> > & getlog() { return app.log; };
};

/**********************************************************************/
