/**********************************************************************/
/* SimMc: Simulator for Many-Core Architecture Research Environment   */
/* Copyright (C) 2010-2013 Arch Lab. Tokyo Institute of Technology    */
/*  Licensed under GPL. See COPYING file for more information.        */
/**********************************************************************/
#pragma once

#include "core/float.h"
#include "network/incc.h"
#include "network/router.h"
#include "common/inttypes.h"
#include "common/config.h"
#include "common/mainmemory.h"
#include "common/simloader.h"
#include "smart/smartunit.h"
#include <vector>

/**********************************************************************/
class Env {
 private:
    ullint cycle;

    void   usage();
    ullint gettime();
    double readargnum(char *);
    bool   readargs (int argc, char **argv);
    void   writeapp(MipsFP &c, MainMemory &m, SimLoader &ld,
                    uint032_t heapptr, uint032_t globalptr, std::string name);
    bool   init(int argc, char **argv);
    bool   running();
    void   sim();

 public:
    typedef enum {
        PATH,
        COMPUTATION,
        MEMORY,
        SERVER,
    } nodetype_t;

    static const int    NUM_CHANNEL_DEF = 3;
    static const int    BUFFER_LENGTH_DEF = 8;
    static const int    SIZE_X_DEF = 4;
    static const int    SIZE_Y_DEF = 4;
    static const ullint MAX_CYCLE_DEF = UINT64_MAX;
    static const ullint SIZE_NODE_MEMORY_DEF = 0x10000000; // 256MB
    static const ullint SIZE_MAIN_MEMORY_DEF = 0x80000000; //   2GB

    std::vector<MipsFP>      core;
    std::vector<MainMemory>  mem;
    std::vector<Incc>        incc;
    std::vector<Router>      router;
    std::vector<SmartUnit>   smart;
    SystemConfig             sc;
    std::string              configfile;     // --config
    uint                     x_rank, y_rank; // --x-rank --y-rank
    bool                     add_server;     // --(no-)server
    bool                     add_memory;     // --memory
    bool                     add_smart;      // --smart
    std::string              smart_pattern;  // --smart-pattern
    ullint                   max_cycle;      // --end-cycle
    std::vector<std::string> binaryfiles;
    int                      num_core;       // # of running phy. core

    Env();
    int sim_main(int argc, char **argv);
};

/**********************************************************************/
