/**********************************************************************/
/* SimMc: Simulator for Many-Core Architecture Research Environment   */
/* Copyright (C) 2010-2013 Arch Lab. Tokyo Institute of Technology    */
/*  Licensed under GPL. See COPYING file for more information.        */
/**********************************************************************/
#include "allocator.h"

/* Arbiter (width x width, iSlip or fixed priority)                   */
/**********************************************************************/
void Arbiter::init(int width, bool fix_priority)
{
    this->width = width;
    this->fix_priority = fix_priority;
    lastgrant = width - 1;
}

/**********************************************************************/
void Arbiter::arbitrate(std::vector<int> &request, std::vector<int> &grant)
{
    int granted = 0;
    for (int i = 0; i < width; i++) {
        int index = (lastgrant + i + 1) % width;
        grant[index] = request[index] && !granted;
        granted |= grant[index];
    }
}

/**********************************************************************/
void Arbiter::setwinner(std::vector<int> &grant)
{
    if (fix_priority)
        return;
    for (int i = 0; i < width; i++)
        if (grant[i])
            lastgrant = i;
}

/* Allocator (width x width)                                          */
/**********************************************************************/
void Allocator::init(int width, bool fix_priority)
{
    this->width = width;
    arb_in.resize(width);
    arb_out.resize(width);
    tr.assign (width, std::vector<int>(width, 0));
    mid.assign(width, std::vector<int>(width, 0));
    for (int i = 0; i < width; i++) {
        arb_in[i].init (width, fix_priority);
        arb_out[i].init(width, fix_priority);
    }
}

/**********************************************************************/
void Allocator::transpose(std::vector<std::vector<int> > &a,
                          std::vector<std::vector<int> > &b)
{
    for (int i = 0; i < width; i++)
        for (int j = 0; j < width; j++)
            b[i][j] = a[j][i];
}

/**********************************************************************/
void Allocator::allocate(std::vector<std::vector<int> > &request,
                         std::vector<std::vector<int> > &grant)
{
    transpose(request, tr);
    for (int i = 0; i < width; i++)
        arb_out[i].arbitrate(tr[i], mid[i]);
    transpose(mid, tr);
    for (int i = 0; i < width; i++)
        arb_in[i].arbitrate(tr[i], grant[i]);
    transpose(grant, tr);
    for (int i = 0; i < width; i++) {
        arb_in[i].setwinner(grant[i]);
        arb_out[i].setwinner(tr[i]);
    }
}

/**********************************************************************/
