/**********************************************************************/
/* SimMc: Simulator for Many-Core Architecture Research Environment   */
/* Copyright (C) 2010-2013 Arch Lab. Tokyo Institute of Technology    */
/*  Licensed under GPL. See COPYING file for more information.        */
/**********************************************************************/
#pragma once

#include "packet.h"
#include "flow.h"
#include "../common/config.h"
#include <vector>

/* Interface of Routers and INCCs.                                    */
/**********************************************************************/
class Connectable {
 protected:
    int num_inbuf;
 public:
    std::vector<Flit*>        out_flit, in_flit;
    std::vector<FlowControl*> out_flow, in_flow;

    Connectable() {}
    virtual ~Connectable() { release(); }
    virtual void init (int num_port, SystemConfig &sc) {
        release();
        this->num_inbuf = sc.num_inbuf;
        out_flit.resize(num_port, NULL);
        in_flit.resize(num_port, NULL);
        out_flow.resize(num_port, NULL);
        in_flow.resize(num_port, NULL);
        for (int i = 0; i < num_port; i++) {
            out_flit[i] = new Flit();
            in_flow[i]  = new FlowControl();
            in_flow[i]->init(sc.num_channel, sc.num_inbuf);
        }
    }
    virtual void release () {
        std::vector<Flit*>::iterator it1;
        for (it1 = out_flit.begin(); it1 != out_flit.end(); it1++)
            delete *it1;
        std::vector<FlowControl*>::iterator it2;
        for (it2 = in_flow.begin(); it2 != in_flow.end(); it2++)
            delete *it2;
    }
    virtual void connect (int srcport, Connectable &destunit, int destport) {
        in_flit[srcport]  = destunit.out_flit[destport];
        out_flow[srcport] = destunit.in_flow[destport];
    }

    // Flow Control
    bool accepthead (int port, int ch) {
        if (out_flow[port] == NULL) return true;
        return out_flow[port]->accepthead(ch);
    }
    bool acceptbody (int port, int ch) {
        if (out_flow[port] == NULL) return true;
        return out_flow[port]->acceptbody(ch);
    }
    void inccredit (int port, int ch) { in_flow[port]->inccredit(ch); }
    void deccredit (int port, int ch)
        { if (out_flow[port]) out_flow[port]->deccredit(ch); }
    void turnoff   (int port, int ch) { in_flow[port]->turnoff(ch); }
    void turnon    (int port, int ch) { in_flow[port]->turnon(ch); }
    void setcredit (int port, int ch, int value)
        { in_flow[port]->setcredit(ch, value); }
    int  getcredit (int port, int ch) {
        if (out_flow[port]) return out_flow[port]->getcredit(ch);
        return num_inbuf;
    }            
};

/**********************************************************************/
