/**********************************************************************/
/* SimMc: Simulator for Many-Core Architecture Research Environment   */
/* Copyright (C) 2010-2013 Arch Lab. Tokyo Institute of Technology    */
/*  Licensed under GPL. See COPYING file for more information.        */
/**********************************************************************/
#pragma once

#include <vector>

/**********************************************************************/
class FlowControl {
 private:
    int full;
    std::vector<int> credit;

 public:
    void init (int num_channel, int full) {
        this->full = full;
        credit.resize(num_channel, full);
    }
    // both credit-based and Xon-Xoff
    bool accepthead (int ch = 0) { return credit[ch] == full; }
    bool acceptbody (int ch = 0) { return credit[ch] != 0; }
    // credit-based
    void inccredit (int ch = 0)
        { credit[ch] = (credit[ch] == full) ? full : credit[ch] + 1; }
    void deccredit (int ch = 0)
        { credit[ch] = (credit[ch] == 0   ) ? 0    : credit[ch] - 1; }
    // Xon-Xoff
    void turnoff (int ch = 0) { credit[ch] = 0; }
    void turnon  (int ch = 0) { credit[ch] = full; }
    // for SmartCore Unit
    void setcredit (int ch, int value) 
        { credit[ch] = (value > full) ? full : (value < 0) ? 0 : value; }
    void setcredit (int value) { setcredit(0, value); }
    int  getcredit (int ch = 0) { return credit[ch]; }
};

/**********************************************************************/
