/**********************************************************************/
/* SimMc: Simulator for Many-Core Architecture Research Environment   */
/* Copyright (C) 2010-2013 Arch Lab. Tokyo Institute of Technology    */
/*  Licensed under GPL. See COPYING file for more information.        */
/**********************************************************************/
#pragma once

#include "connectable.h"
#include "../common/mainmemory.h"
#include "../common/config.h"
#include <map>

class Incc;

enum {
    DMA_NA     = 0, 
    DMA_PUT    = 1,
    DMA_PUT64  = 2,
    DMA_GET    = 3,
    DMA_ACK    = 4,
    DMAFLAG_ACK  = 1,
    DMAFLAG_NACK = 2,
};

/* Inter-node Communication Controller (INCC),                        */
/* which consists of InccSend (to NoC) and InccRecv (from NoC)        */
/**********************************************************************/
class DmaCommand {
 public:
    int       type;
    uint032_t remote_id;
    uint064_t remote_addr, local_addr;
    uint032_t remote_stride, local_stride;
    uint032_t immediate;
    uint032_t size, transfered;
    uint      channel;

    DmaCommand() { clear(); }
    void clear () {
        type = DMA_NA;
        remote_id = remote_stride = local_stride = 0;
        remote_addr = local_addr = 0;
        immediate = 0;
        size = transfered = 0;
        channel = 0;
    }
    bool valid () { return (type != DMA_NA); }
};

/**********************************************************************/
class InccSend {
 private:
    static const int MAX_ACCEPT_GET = 2;

    Flit        *out_flit;
    FlowControl *out_flow;
    MainMemory  *mem;
    std::map<uint, uint> lpmap;

    uint my_x, my_y, my_id, num_channel;
    bool laddr_64bit, lookahead_rc;
    int type, count, next_type, next_count;
    std::vector<DmaCommand> get_cmd, ack_cmd;
    DmaCommand cur_cmd, put_cmd, next_cur_cmd, next_put_cmd;
    DmaCommand new_get_cmd, new_ack_cmd;
    bool deq_get_cmd, deq_ack_cmd;
    ullint num_sent_flit, cycle;

    void addputcommand();
    void addgetcommand();
    void getnextcommand();
    void generateflit();

 public:
    InccSend() {}
    void init(uint id, SystemConfig &sc, MainMemory *mem);
    void connect(Incc *incc);
    void drive();
    void update();
    bool isbusy();
};

/**********************************************************************/
class InccRecv {
 private:
    Flit        *in_flit;
    FlowControl *in_flow;
    Flit         flitbuf;
    MainMemory  *mem;

    uint num_channel;
    std::vector<int> type, count, next_type, next_count;
    std::vector<DmaCommand> cmd, next_cmd;

    void readflit();

 public:
    InccRecv() {}
    void init(SystemConfig &sc, MainMemory *mem);
    void connect(Incc *incc);
    void drive();
    void update();
    bool isbusy();
};

/**********************************************************************/
class Incc : public Connectable {
 private:
    MainMemory *mem;
    ullint cycle;
    uint num_channel;
    bool credit_fc;
    
 public:
    InccSend send;
    InccRecv recv;

    void init(uint pid, SystemConfig &sc, MainMemory *mem);
    void connect(int srcport, Connectable &destunit, int destport);
    void drive();
    void update();
    bool isbusy() { return send.isbusy() || recv.isbusy(); }
};

/**********************************************************************/
