/**********************************************************************/
/* SimMc: Simulator for Many-Core Architecture Research Environment   */
/* Copyright (C) 2010-2013 Arch Lab. Tokyo Institute of Technology    */
/*  Licensed under GPL. See COPYING file for more information.        */
/**********************************************************************/
#pragma once

#include "../common/inttypes.h"
#include <iostream>
#include <iomanip>

/**********************************************************************/
enum {
    DIR_N = 0,
    DIR_E = 1,
    DIR_S = 2,
    DIR_W = 3,
    DIR_C = 4,
    NUM_DIR = 5,
    MAX_VC  = 8,
    MAX_DIR_X_VC = NUM_DIR * MAX_VC,
};

#define opposite(x) ((x) == DIR_N ? DIR_S :     \
                    ((x) == DIR_E ? DIR_W :     \
                    ((x) == DIR_S ? DIR_N :     \
                    ((x) == DIR_W ? DIR_E : DIR_C))))

/**********************************************************************/
class Flit {
 public:
    static const int MAX_FLIT_WORDS = 4;

    bool valid, head, tail;
    int indir, outdir, inch, outch, nextoutdir, inclass, outclass;
    uint032_t data[MAX_FLIT_WORDS];
    bool data_valid[MAX_FLIT_WORDS];

    ullint id, timer;

    Flit() { invalidate(); }
    void clear () {
        valid = head = tail = false;
        indir = outdir = inch = outch = nextoutdir = 0;
        inclass = outclass = 0;
        for (int i = 0; i < MAX_FLIT_WORDS; i++) {
            data[i] = 0;
            data_valid[i] = false;
        }
        id = timer = 0;
    }
    void invalidate () { valid = false; }
    void print () {
        if (! valid) {
            std::cout << "[---]" << std::endl;
            return;
        }
        std::cout << '[' << ((valid) ? 'v' : '-');
        std::cout << ((head) ? 'h' : '-') << ((tail) ? 't' : '-') << ' ';
        std::cout << indir << ':' << inch << " > ";
        std::cout << outdir << ':' << outch << " >> ";
        std::cout << nextoutdir << ' ' << std::hex;
        for (int i = 0; i < MAX_FLIT_WORDS; i++) {
            if (data_valid[i])
                std::cout << std::setw(8) << std::setfill('0') << data[i];
            else
                std::cout << "--------";
            std::cout << ' ';
        }
        std::cout << "id:" << std::dec << id << ' ';
        std::cout << "tm:" << timer << ']' << std::endl;
    }
};

/**********************************************************************/
enum {
    FT_HEADER = 1,
    FT_ADDR_L = 2,
    FT_ADDR_H = 3,
    FT_STRIDE = 4,
    FT_DATA   = 5,
    FT_IMM    = 6,
    FT_END    = -1,
};

/**********************************************************************/
class Packet {
 public:
    static const int MAX_FLIT = 11;
    enum {
        PUT32         = 0,
        PUT32_STRIDE  = 1,
        PUT64         = 2,
        PUT32_ACKNACK = 3,
        NUM_TYPE      = 4,
    };

    static const int def[NUM_TYPE][MAX_FLIT + 1][Flit::MAX_FLIT_WORDS];
};

/**********************************************************************/
