/**********************************************************************/
/* SimMc: Simulator for Many-Core Architecture Research Environment   */
/* Copyright (C) 2010-2013 Arch Lab. Tokyo Institute of Technology    */
/*  Licensed under GPL. See COPYING file for more information.        */
/**********************************************************************/
#include "packetgen.h"
#include "routecomp.h"
#include <stdlib.h>

/**********************************************************************/
PacketGen::PacketGen()
{
    cycle      = 0;
    warm_cycle = WARM_UP_CYCLE_DEF;
    main_cycle = WARM_UP_CYCLE_DEF + SIM_CYCLE_DEF;
    num_send = num_recv = num_recv_main = sum_latency = 0;
}

/**********************************************************************/
void PacketGen::init(uint id, SystemConfig &sc)
{
    Connectable::init(1, sc);
    this->id = id;
    num_channel  = sc.num_channel;
    lookahead_rc = sc.lookahead_rc;
    credit_fc    = sc.credit_fc;
}

/**********************************************************************/
void PacketGen::addpacket(uint dest, int length, int ch)
{
    for (int i = 0; i < length; i++) {
        Flit newflit;
        newflit.valid = true;
        newflit.head  = (i == 0);
        newflit.tail  = (i == length - 1);
        newflit.id    = id;
        newflit.timer = cycle;
        if (i == 0) {
            newflit.data[0] = dest;
            if (lookahead_rc)
                newflit.nextoutdir = RouteComp::getdir(id, dest);
        }
        newflit.outclass = newflit.outch = ch;
        send_queue.push_back(newflit);
    }
    if (cycle >= warm_cycle && cycle < main_cycle)
        num_send++;
}

/**********************************************************************/
void PacketGen::drive()
{
    // sender
    out_flit[0]->clear();
    if (! send_queue.empty()) {
        int  ch      = send_queue.front().outch;
        bool send_ok = (send_queue.front().head) ? accepthead(0, ch) :
                                                   acceptbody(0, ch);
        if (send_ok) {
            *out_flit[0] = send_queue.front();
            send_queue.erase(send_queue.begin());
        }
    }
}

/**********************************************************************/
void PacketGen::update()
{
    cycle++;
    // sender
    if (credit_fc && out_flit[0]->valid)
        deccredit(0, out_flit[0]->outch);

    // receiver
    Flit flitbuf = *in_flit[0];
    if (flitbuf.valid && flitbuf.head) {
        if (flitbuf.data[0] != id) {
            std::cout << "FATAL: a packet went a wrong destination. ";
            std::cout << "id:" << std::hex << id;
            std::cout << ", src:" << flitbuf.id;
            std::cout << ", dest:" << flitbuf.data[0] << std::endl;
            exit(1);
        }
    }
    if (flitbuf.valid && flitbuf.tail) {
        if (cycle >= warm_cycle && cycle < main_cycle)
            num_recv_main++;
        if (flitbuf.timer >= warm_cycle && flitbuf.timer < main_cycle) {
            num_recv++;
            sum_latency += (cycle - flitbuf.timer);
        }
    }

    // always receive flits
    for (int i = 0; i < num_channel; i++)
        turnon(0, i);
}

/**********************************************************************/
