/**********************************************************************/
/* SimMc: Simulator for Many-Core Architecture Research Environment   */
/* Copyright (C) 2010-2013 Arch Lab. Tokyo Institute of Technology    */
/*  Licensed under GPL. See COPYING file for more information.        */
/**********************************************************************/
#pragma once

#include "network/packetgen.h"
#include "network/router.h"
#include "smart/smartunit.h"

/**********************************************************************/
class Env {
 private:
    void   usage();
    bool   wrong_option(char *);
    double readargnum(char *);
    bool   readargs (int argc, char **argv);
    double network_capacity (int x, int y, bool smart);
    double sim (double rate);

 public:
    typedef enum {
        RANDOM,
        COMPLEMENT,
        TORNADO,
    } pattern_t;

    static const double RATE_INTERVAL;
    static const double END_RATE;
    static const int NUM_CHANNEL_DEF = 2;
    static const int BUFFER_LENGTH_DEF = 4;
    static const int PACKET_LENGTH_DEF = 5;
    static const int SIZE_X_DEF = 8;
    static const int SIZE_Y_DEF = 8;
    static const pattern_t TRAFFIC_DEF = RANDOM;

    std::vector<PacketGen> core;
    std::vector<Router>    router;
    std::vector<SmartUnit> smart;
    SystemConfig           sc;
    int                    packet_length;
    std::string            pattern_str;
    pattern_t              pattern;
    double                 rate_interval;
    double                 rate_end;
    bool                   use_smart;
    std::string            smart_pattern_str;
    bool                   use_normalized;
    bool                   end_saturated;
    bool                   end_once;

    Env();
    int sim_main(int argc, char **argv);
};

/**********************************************************************/
