/**********************************************************************/
/* SimMc: Simulator for Many-Core Architecture Research Environment   */
/* Copyright (C) 2010-2013 Arch Lab. Tokyo Institute of Technology    */
/*  Licensed under GPL. See COPYING file for more information.        */
/**********************************************************************/
#pragma once

#include "../network/connectable.h"
#include "../network/packet.h"
#include "../network/routecomp.h"
#include "../network/router.h"
#include "../common/config.h"
#include <vector>

/* SmartCore Unit for MF Router, connected between INCC and router    */
/**********************************************************************/
class SmartUnit : public Connectable {
 private:
    static const int SU_BUFFER_LENGTH_DEF = 12;
    static const int SU_TIMEOUT_DEF = 10000;
    enum {
        CH_NORMAL   = 0,
        CH_MERGE    = 1,
        CH_COPY     = 2,
    };
    Router *rt;
    bool master_mode, slave_mode;
    std::vector<Flit> vbuf_m, vbuf_s;
    uint my_x, my_y, my_id;
    uint pair_x, pair_y, pair_id;
    int  num_channel, num_inbuf, num_inbuf_rt;
    bool lookahead_rc, credit_fc;

    FlowControl flow_m, flow_s;
    Flit copy_flit;
    bool vbuf_deq;
    int  inch;
    bool inch_lock;
    int  timeout_cycle, elapsed_cycle;
    ullint total_cycle;

    int getcredit_master(int ch, FlowControl &flow);
    void drive_master();
    void drive_slave();
    void drive_normal();
    void update_master();
    void update_slave();
    void update_normal();

 public:
    enum {
        SIDE_INCC = 0,
        SIDE_RT   = 1,
    };

    bool detect_error;

    void init (uint pid, SystemConfig &sc, Router *rt);
    void drive();
    void update();
    void print();
};

/**********************************************************************/
