/**********************************************************************/
/* SimMc: Simulator for Many-Core Architecture Research Environment   */
/* Copyright (C) 2010-2013 Arch Lab. Tokyo Institute of Technology    */
/*  Licensed under GPL. See COPYING file for more information.        */
/**********************************************************************/
#include "../../sim/core/float.h"
#include "../../sim/common/simloader.h"
#include "../../sim/common/mainmemory.h"

#include <iostream>

const char *loadfiles[102] = {
    "int/001.li.elf",              "int/002.add.elf",             
    "int/003.and.elf",             "int/004.beq.elf",             
    "int/005.beql.elf",            "int/006.bge.elf",             
    "int/007.bgez.elf",            "int/008.bgezal.elf",          
    "int/009.bgezl.elf",           "int/010.bgt.elf",             
    "int/011.bgtz.elf",            "int/012.bgtzl.elf",           
    "int/013.ble.elf",             "int/014.blez.elf",            
    "int/015.blezl.elf",           "int/016.blt.elf",             
    "int/017.bltz.elf",            "int/018.bltzal.elf",          
    "int/019.bltzall.elf",         "int/020.bltzl.elf",           
    "int/021.bne.elf",             "int/022.bnel.elf",            
    "int/023.clo.elf",             "int/024.clz.elf",             
    "int/025.div.elf",             "int/026.divu.elf",            
    "int/027.jump.elf",            "int/028.load_byte.elf",       
    "int/029.load_half.elf",       "int/030.load_word.elf",       
    "int/031.load_word_l.elf",     "int/032.load_word_r.elf",     
    "int/033.lui_la.elf",          "int/034.madd.elf",            
    "int/035.maddu.elf",           "int/036.mov.elf",             
    "int/037.msub.elf",            "int/038.msubu.elf",           
    "int/039.mult.elf",            "int/040.multu.elf",           
    "int/041.nor.elf",             "int/042.or.elf",              
    "int/043.shift.elf",           "int/044.slt.elf",             
    "int/045.sltu.elf",            "int/046.sub.elf",             
    "int/047.store_word_lr.elf",   "int/048.xor.elf",              
    "float/101.abs_neg.elf",       "float/102.add_s.elf",         
    "float/103.add_d.elf",         "float/104.cvt.elf",           
    "float/105.div_s.elf",         "float/106.div_d.elf",         
    "float/107.loadstore.elf",     "float/108.madd_nmadd.elf",    
    "float/109.mfc1.elf",          "float/110.mov_fpu.elf",       
    "float/111.msub_nmsub.elf",    "float/112.mul_s.elf",         
    "float/113.mul_d.elf",         "float/114.recip.elf",         
    "float/115.to_int.elf",        "float/116.rsqrt.elf",         
    "float/117.sqrt_s.elf",        "float/118.sqrt_d.elf",        
    "float/119.sub_s.elf",         "float/120.sub_d.elf",         
    "float/121.bc1f_eq_s.elf",     "float/122.bc1f_eq_d.elf",     
    "float/123.bc1fl_eq_s.elf",    "float/124.bc1fl_eq_d.elf",    
    "float/125.bc1t_eq_s.elf",     "float/126.bc1t_eq_d.elf",     
    "float/127.bc1tl_eq_s.elf",    "float/128.bc1tl_eq_d.elf",    
    "float/129.bc1f_le_s.elf",     "float/130.bc1f_le_d.elf",     
    "float/131.bc1fl_le_s.elf",    "float/132.bc1fl_le_d.elf",    
    "float/133.bc1t_le_s.elf",     "float/134.bc1t_le_d.elf",     
    "float/135.bc1tl_le_s.elf",    "float/136.bc1tl_le_d.elf",    
    "float/137.bc1f_lt_s.elf",     "float/138.bc1f_lt_d.elf",     
    "float/139.bc1fl_lt_s.elf",    "float/140.bc1fl_lt_d.elf",    
    "float/141.bc1t_lt_s.elf",     "float/142.bc1t_lt_d.elf",     
    "float/143.bc1tl_lt_s.elf",    "float/144.bc1tl_lt_d.elf",    
    "float/145.f_nge_ngl_s.elf",   "float/146.f_nge_ngl_d.elf",   
    "float/147.ngt_cle_olt_s.elf", "float/148.ngt_cle_olt_d.elf", 
    "float/149.seq_sf_ueq_s.elf",  "float/150.seq_sf_ueq_d.elf",  
    "float/151.ule_ult_un_s.elf",  "float/152.ule_ult_un_d.elf",  
    "float/153.ole_s.elf",         "float/154.ole_d.elf"          
};

/**********************************************************************/
int main (int argc, char** argv)
{
    MipsFP core;
    MainMemory mem;
    SimLoader ld;
    if (argc == 1) {
        std::cout << "!! no arguments" << std::endl;
        return 1;
    }

    if (chdir(argv[1])) {
        std::cout << "!! failed to chdir: " << argv[0] << std::endl;
        return 1;
    }

    core.init(&mem);
    mem.init(0x8000000);
    for (int app = 0; app < 102; app++) {
        const char *file = loadfiles[app];
        if (ld.loadfile(file)) {
            std::cout << "!! failed to load: " << file << std::endl;        
            return 1;
        }
        
        for (int i = 0; i < ld.memtabnum; i++) {
            mem.write(ld.memtab[i].addr, 1, ld.memtab[i].data);
            if (core.ss->heapptr < ld.memtab[i].addr)
                core.ss->heapptr = ld.memtab[i].addr;
        }
        for (int i = 0; i < NREG; i++)
            core.as->r[i] = 0;
        for (int i = 0; i < NFREG; i++)
            core.asfp->fr.w[i] = 0;
        core.as->r[REG_SP] = 0x7fff000;
        core.as->pc = core.as->r[REG_T9] = ld.entry;
        core.as->delay_npc = 0;
        for (int i = 0; i < ld.symtabnum; i++)
            if (strcmp(ld.symtab[i].name, "_gp") == 0) {
                core.as->r[REG_GP] = ld.symtab[i].addr;
                break;
            }
        core.state = CPU_START;
        while (core.running()) {
            core.drive();
            core.update();
        }
        std::cout << std::endl << std::endl;
    }
    return 0;
}

/**********************************************************************/










