/**********************************************************************/
/* SimMc: Simulator for Many-Core Architecture Research Environment   */
/* Copyright (C) 2010-2013 Arch Lab. Tokyo Institute of Technology    */
/*  Licensed under GPL. See COPYING file for more information.        */
/**********************************************************************/
#include "../../sim/network/incc.h"
#include "../../sim/common/mainmemory.h"
#include "../../sim/common/config.h"
#include "../../sim/common/memorymap.h"
#include <iostream>

Incc incc[2];
MainMemory mem[2];
SystemConfig sc;

/**********************************************************************/
void dma_wait()
{
    int cnt = 0;
    while ((incc[0].isbusy() || incc[1].isbusy()) && cnt < 1000) {
        cnt++;
        incc[0].drive();
        incc[1].drive();
        incc[0].update();
        incc[1].update();
        if (incc[0].out_flit[0]->valid) {
            std::cout << "> ";
            incc[0].out_flit[0]->print();
        }
        if (incc[1].out_flit[0]->valid) {
            std::cout << "< ";
            incc[1].out_flit[0]->print();
        }
    }
    std::cout << std::endl;
}

/**********************************************************************/
void dma_put(int src, ullint laddr, ullint raddr,
             uint lstride, uint rstride, uint size)
{
    if (src == 0) { // PUT32
        mem[0].write(SC_MMAP_DMA_PUT_REMOTE_ID    , 4, 0x101);
        mem[0].write(SC_MMAP_DMA_PUT_REMOTE_ADDR  , 4, raddr);
        mem[0].write(SC_MMAP_DMA_PUT_REMOTE_STRIDE, 4, rstride);
        mem[0].write(SC_MMAP_DMA_PUT_LOCAL_ADDR   , 4, laddr);
        mem[0].write(SC_MMAP_DMA_PUT_LOCAL_STRIDE , 4, lstride);
        mem[0].write(SC_MMAP_DMA_PUT_SIZE         , 4, size);
        mem[0].write(SC_MMAP_DMA_PUT_CMD          , 4, DMA_PUT);
    } else {        // PUT64
        mem[1].write(SC_MMAP_DMA_PUT_REMOTE_ID    , 4, 0x000);
        mem[1].write(SC_MMAP_DMA_PUT_REMOTE_ADDR  , 4, raddr);
        mem[1].write(SC_MMAP_DMA_PUT_REMOTE_STRIDE, 4, raddr >> 32);
        mem[1].write(SC_MMAP_DMA_PUT_LOCAL_ADDR   , 4, laddr);
        mem[1].write(SC_MMAP_DMA_PUT_LOCAL_STRIDE , 4, lstride);
        mem[1].write(SC_MMAP_DMA_PUT_SIZE         , 4, size);
        mem[1].write(SC_MMAP_DMA_PUT_CMD          , 4, DMA_PUT64);
    }
    int dest = 1 - src;
    std::cout << "PUT [" << src << ":0x" << std::hex << laddr;
    std::cout << '/' << std::dec << lstride << "]";
    std::cout << " > [" << dest << ":0x" << std::hex << raddr;
    std::cout << '/' << std::dec << rstride << "]";
    std::cout << " size: " << size << std::endl;
    dma_wait();
}

/**********************************************************************/
void dma_get(int src, ullint laddr, ullint raddr,
             uint lstride, uint rstride, uint size)
{
    if (src == 0) { // GET32
        mem[0].write(0x800, 4, 0x000);
        mem[0].write(0x804, 4, raddr);
        mem[0].write(0x808, 4, rstride);
        mem[0].write(0x80c, 4, laddr);
        mem[0].write(0x810, 4, lstride);
        mem[0].write(0x814, 4, size);
        mem[0].write(0x818, 4, 0);
        mem[0].write(0x81c, 4, DMA_PUT);
    } else {        // GET64
        mem[1].write(0x800, 4, 0x101);
        mem[1].write(0x804, 4, laddr);
        mem[1].write(0x808, 4, lstride);
        mem[1].write(0x80c, 4, raddr);
        mem[1].write(0x810, 4, raddr >> 32);
        mem[1].write(0x814, 4, size);
        mem[1].write(0x818, 4, 0);
        mem[1].write(0x81c, 4, DMA_PUT64);
    }
    int dest = 1 - src;
    std::cout << "GET [" << src << ":0x" << std::hex << laddr;
    std::cout << '/' << std::dec << lstride << "]";
    std::cout << " < [" << dest << ":0x" << std::hex << raddr;
    std::cout << '/' << std::dec << rstride << "]";
    std::cout << " size: " << size << std::endl;
    dma_put(src, 0x800, SC_MMAP_DMA_GET_REMOTE_ID, 4, 4, 32);
}

/**********************************************************************/
int main (void)
{
    if (! sc.readconfigfile("map.txt")) {
        std::cerr << "!! Error reading config" << std::endl;
        return 1;
    }
    sc.lookahead_rc = true;
    sc.num_channel = 2;

    mem[0].init(0x200000000ull);
    mem[1].init(0x10000);
    incc[0].init(0x0101, sc, &mem[0]);
    incc[1].init(0x0202, sc, &mem[1]);
    incc[0].connect(0, incc[1], 0);
    incc[1].connect(0, incc[0], 0);
    for (uint addr = 0x1000; addr < 0x1100; addr += 4) {
        mem[0].write(addr, 4, addr);
        mem[1].write(addr, 4, addr);
    }
    mem[0].write(SC_MMAP_DMA_GET_CH, 4, 1);
    mem[1].write(SC_MMAP_DMA_GET_CH, 4, 1);

    dma_put(0, 0x1000, 0x2000, 4, 4, 40);
    dma_put(0, 0x1000, 0x3000, 16, 8, 40);
    dma_put(1, 0x1000, 0x2000, 4, 4, 40);
    dma_put(1, 0x1000, 0x100000000ull, 16, 4, 40);
    dma_get(0, 0x4000, 0x1000, 4, 4, 40);
    dma_get(0, 0x5000, 0x1000, 16, 8, 40);
    dma_get(1, 0x4000, 0x100000000ull, 4, 4, 40);
    dma_get(1, 0x5000, 0x1000, 16, 4, 40);

    std::cout << std::endl;
    mem[0].print();
    mem[1].print();
    return 0;
}

/**********************************************************************/










