/**********************************************************************/
/* SimMc: Simulator for Many-Core Architecture Research Environment   */
/* Copyright (C) 2010-2013 Arch Lab. Tokyo Institute of Technology    */
/*  Licensed under GPL. See COPYING file for more information.        */
/**********************************************************************/
#include "../../sim/common/inttypes.h"
#include "../../sim/common/mainmemory.h"
#include <iostream>

MainMemory mem;

/**********************************************************************/
void writetest(uint064_t addr, int size, uint064_t data)
{
    int result;
    result = mem.write(addr, size, data);
    std::cout << "[W] addr:" << std::hex << addr << ", ";
    std::cout <<     "size:" << std::hex << size << ", ";
    std::cout <<     "data:" << std::hex << data << ", ";
    std::cout <<   "result:" << std::dec << result << std::endl;
}

/**********************************************************************/
void readtest(uint064_t addr, int size)
{
    uint064_t data = 0;
    int result;
    if (size == 1) {
        uint008_t tmp = 0;
        result = mem.read(addr, size, &tmp);
        data = tmp;
    } else if (size == 2) {
        uint016_t tmp = 0;
        result = mem.read(addr, size, &tmp);
        data = tmp;
    } else if (size == 4) {
        uint032_t tmp = 0;
        result = mem.read(addr, size, &tmp);
        data = tmp;
    } else /*if (size == 8)*/ {
        result = mem.read(addr, size, &data);
    }
    std::cout << "[R] addr:" << std::hex << addr << ", ";
    std::cout <<     "size:" << std::hex << size << ", ";
    std::cout <<     "data:" << std::hex << data << ", ";
    std::cout <<   "result:" << std::dec << result << std::endl;
}

/**********************************************************************/
int main (void)
{
    mem.init(0x200000000ull);
    // write test
    writetest(0x1000, 1, 0x12);
    writetest(0x1001, 1, 0x34);
    writetest(0x1002, 1, 0x56);
    writetest(0x1003, 1, 0x78);
    writetest(0x1010, 2, 0x1234);
    writetest(0x1012, 2, 0x5678);
    writetest(0x1020, 4, 0x12345678);
    writetest(0x1024, 4, 0x9abcdef0);
    writetest(0x1030, 8, 0x123456789abcdef0ull);
    writetest(0x1fffffffcull, 4, 0x12345678);
    writetest(0x200000000ull, 4, 0x12345678);
    mem.update();

    // read test
    readtest(0x1000, 8);
    readtest(0x1010, 8);
    readtest(0x1020, 8);
    readtest(0x1030, 1);
    readtest(0x1031, 1);
    readtest(0x1032, 1);
    readtest(0x1033, 1);
    readtest(0x1034, 2);
    readtest(0x1036, 2);
    readtest(0x1038, 4);
    readtest(0x100000000ull, 4);
    readtest(0x1fffffffcull, 4);
    readtest(0x200000000ull, 4);
    return 0;
}

/**********************************************************************/










